import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.imageio.ImageIO;

public class Mask2 {

	public static void main(String[] args) {

		// 画像読み込み
		System.out.print("Input 'bmp' file name?: ");
		String fileName = in();
		BufferedImage img = readImg(fileName);
		int width = img.getWidth();
		int height = img.getHeight();

		System.out.println("Image width/height=" + width + "/" + height);

		// ウインドウ作成
		MainFrame frame = new MainFrame("Chromakey", width * 2, height * 2);
		frame.setVisible(true);

		// ウインドウに入力画像表示
		frame.addImage(img);

		// ヒストグラム作成
		Histogram histo = new Histogram(HistoFrame.HIST_MAX);

		// 画像の各画素のHSV情報を格納する配列
		HSV[] hsvDatas = new HSV[width * height];

		// HSV変換+ヒストグラム作成
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				Color color = new Color(img.getRGB(x, y));
				HSV hsv = RGB2HSV.rgb2hsv(color);
				hsvDatas[(height - (y + 1)) * width + x] = hsv;

				int tmph = Math.round(hsv.getH());
				if (tmph < 360) {
					histo.addCount(tmph);
				}
			}
		}

		// ヒストグラムウインドウ表示
		HistoFrame histoFrame = new HistoFrame("Histogram", histo);
		histoFrame.setVisible(true);

		// しきい値入力
		System.out.println("Thresholding values: t1 and t2 ?");
		System.out.print("t1:");
		int t1 = Integer.parseInt(in());
		System.out.print("t2:");
		int t2 = Integer.parseInt(in());

		// マスク画像作成
		BufferedImage maskImg = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int rgb;
				int h = hsvDatas[(height - (y + 1)) * width + x].getH();
				if (t1 <= h && h <= t2) {
					rgb = Color.BLACK.getRGB();
				} else {
					rgb = Color.WHITE.getRGB();
				}
				maskImg.setRGB(x, y, rgb);
			}
		}
		frame.addImage(maskImg);

		// 背景色入力
		System.out.println("Background color [R G B]:");
		System.out.print("R:");
		int red = Integer.parseInt(in());
		System.out.print("G:");
		int green = Integer.parseInt(in());
		System.out.print("B:");
		int blue = Integer.parseInt(in());

		
		// 切り出した画像と背景色を合成した画像を作成
		Color backColor = new Color(red, green, blue);
		BufferedImage cutoutImg = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int rgb;
				if (maskImg.getRGB(x, y) == Color.BLACK.getRGB()) {
					rgb = backColor.getRGB();
				} else {
					rgb = img.getRGB(x, y);
				}
				cutoutImg.setRGB(x, y, rgb);
			}
		}
		frame.addImage(cutoutImg);

		// 背景画像読み込み
		System.out.println("Open background 'bmp' image file");
		System.out.print("Input backgournd 'bmp' file name? ");
		fileName = in();
		BufferedImage backImg = readImg(fileName);

		// クロマキー合成画像作成
		BufferedImage chromakeyed = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int rgb;
				if (maskImg.getRGB(x, y) == Color.BLACK.getRGB()) {
					rgb = backImg.getRGB(x, y);
				} else {
					rgb = img.getRGB(x, y);
				}
				chromakeyed.setRGB(x, y, rgb);
			}
		}
		frame.addImage(chromakeyed);
	}

	// コマンドラインから文字列の入力を受け付ける
	private static String in() {
		String str = null;
		try {
			BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
			str = input.readLine();
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
		return str;
	}

	// 画像ファイルを読み込む
	private static BufferedImage readImg(String fileName) {
		BufferedImage img = null;
		try {
			img = ImageIO.read(new File(fileName));
		} catch (IOException e) {
			System.err.println(fileName + "：指定された画像ファイルが見つかりません。");
			System.exit(0);
		}
		return img;
	}
}
